
/* NOTE:
  This is an interrupt service routine.
  You can NOT compile this program with Test
  Stack Overflow turned on and get an
  executable file which will operate
  correctly.  Due to the nature of this
  function the formula used to compute
  the number of paragraphs may not
  necessarily work in all cases.  Use with
  care!  Terminate Stay Resident (TSR)
  programs are complex and no other support
  for them is provided.  Refer to the
  MS-DOS technical documentation
  for more information.  */

  /* This program is developed by Garry Freemyer. It is freeware.
     Although it costs nothing, if you find it useful or fun drop me a note.
     I'd like to hear from Ya.
     Write me at:
     5829 Sawmill Rd.
     Paradise, Ca. 95969.
     916-877-7015

     Press LEFT ALT key twice to toggle program ON/OFF

     If you REALLY like it AND feel generous , Donations will be gratefully
     accepted.

*/

#include <dos.h>
#define INTR 9
#define ALTKEY 0x38
#ifdef __cplusplus
#define __CPPARGS ...
#else
#define __CPPARGS
#endif

/* reduce heaplength and stacklength to make a smaller program in memory */
extern unsigned _heaplen = 1024;
extern unsigned _stklen  = 512;

void nosound()
{
    asm {
        push ax
        in al,0x61
        and al,252
        out 0x61,al
        pop ax
        }
}

void interrupt ( *oldhandler)(__CPPARGS);
void interrupt handler(__CPPARGS)
{
   static unsigned const notes[88] =
   {
   65,
   69,
   73,
   77,
   82,
   87,
   92,
   98,
   104,
   110,
   117,
   123,
   131,
   139,
   147,
   156,
   165,
   175,
   185,
   196,
   208,
   220,
   233,
   247,
   262,
   277,
   294,
   311,
   330,
   349,
   370,
   392,
   415,
   440,
   466,
   494,
   523,
   554,
   587,
   622,
   659,
   698, // Left Shift = 42
   740,
   784,
   831,
   880,
   932,
   988
  ,1046,
  1109,
  1175,
  1244,
  1328,
  698, // Right Shift 54
  1397,
  1480,
  1568,
  0, // Caps Lock 58 No sound for Caps Locks.
  1661,
  1760,
  1865,
  1975,
  2093,
  2217,
  2349,
  2489,
  2637,
  2794,
  2960, // Num Lock just clicks Share with f11
  3136, // scroll lock clicks Share with f12
  3322,
  3520,
  3729,
  3951,
  4186, // highest on scale.
  4436,
  4702,
  4984,
  5283,
  5599,          // 80
  5933,
  6286,
  6652, // *
  7046,
  7456,
  7886,
  2960,
  3136
  };
   static unsigned scancode, lastPressedScan;
   static char incode = 0, toggle = 1;

   if (!incode)
   {
        asm {
            push ax
            push bx
            push cx
            push dx
            mov incode,1
            in al,0x60
            mov scancode,ax
        }
        if (scancode == ALTKEY && lastPressedScan == 0x38) // Alt pressed twice.
        {
            toggle = !toggle;
            if (toggle==0) nosound();
        }
        if (toggle)
        {
            if (scancode > 0x58)
            {
                nosound();
            }
            else
            if (scancode != 0x3A)
            {
                long const denom = 1193180;
                unsigned count = denom / notes[scancode-1];
                asm {
                mov al,0xb6
                out 0x43,al
                mov ax,count
                out 0x42,al
                mov al ,ah
                out 0x42,al
                in al,0x61
                or al,3
                out 0x61,al
                }
                }
                }
   if (scancode>0x58) lastPressedScan = scancode - 0x80;
   else lastPressedScan = scancode;
    asm mov incode,0;
    asm pop dx
    asm pop cx
    asm pop bx
    asm pop ax
    }

   oldhandler();
}


int main(void)
{


    oldhandler = getvect(INTR); // Get address of the current Kb int9 handler.
    setvect(INTR, handler); // install the new interrupt handlers

    /* _psp is the starting address of the
       program in memory.  The top of the stack
       is the end of the program.  Using _SS and
       _SP together we can get the end of the
       stack.  You may want to allow a bit of
       saftey space to insure that enough room
       is being allocated ie:
       (_SS + ((_SP + safety space)/16) - _psp)
    */

    keep(0, (_SS + (_SP/16) - _psp));
return 0;
}
